/*\
* lenactions
* A simple image editing library
* Copyrigth 2013 - Hadrien Croubois
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program. If not, see <http://www.gnu.org/licenses/>.
\*/

#ifndef IMAGE_HH
#define IMAGE_HH

#include <algorithm>
#include <vector>

#include <lenactions/pixel.hh>
#include <lenactions/convolution.hh>

namespace lenactions {

	template<typename T>
	class image {

		public:
			image();
			image(const image<T>&) = default;
			image(image<T>&&     ) = default;
			image(std::istream& in);

			void from_stream(std::istream& in);
			void to_stream  (std::ostream& out, format out_format);

			image<T>& operator=(const image<T>&) = default;
			image<T>& operator=(image<T>&&     ) = default;

			pixel<T>&       operator[] (int);
			const pixel<T>& operator[] (int) const;

			std::pair<size_t, size_t> dimensions() const;

			// ┌────────────────────────────────────────────────────────────────────┐
			// │                               TP 02                                │
			// └────────────────────────────────────────────────────────────────────┘
			image<T>& histogram(std::ostream& out);
			image<T>& gamma(T y);
			image<T>& rotate(T y);
			image<T>& normalize();

			// ┌────────────────────────────────────────────────────────────────────┐
			// │                               TP 03                                │
			// └────────────────────────────────────────────────────────────────────┘
			template <size_t H, size_t V> image<T>& compose   (const Kernel<T, H, V>&, const T& norm = T(1));
			static                        image<T>  assemblage(image<T>&, image<T>&, const pixelOperator<T>&);
			// ──────────────────────────────────────────────────────────────────────
			image<T>& smooth         ();
			image<T>& contour_prewitt(const pixelOperator<T>& = lenactions::pixel<T>::quadratic);
			image<T>& contour_sobel  (const pixelOperator<T>& = lenactions::pixel<T>::quadratic);
			image<T>& contour_kirsch ();

			// ┌────────────────────────────────────────────────────────────────────┐
			// │                                                                    │
			// └────────────────────────────────────────────────────────────────────┘

			// void inv();
			// void connected(int adj = 0);
			// float Hseuil(float p) const;
			// image<T> seuil_global(float s = 0.5) const;
			// image<T> seuil_local() const;
			// image<T> seuil_histerisis(float high, float low) const;
			// image<T> affinage() const;

		private:
			size_t o_cols(int64_t j, int64_t offset) const;
			size_t o_rows(int64_t i, int64_t offset) const;

			size_t                m_rows;
			size_t                m_cols;
			std::vector<pixel<T>> m_bitmap;
	};
}

#include "image.hpp"

#endif // IMAGE_H
